@@ -1002,7 +1002,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
           {
             for (x=0; x < (long) image->columns; x++)
             {
-              index=(IndexPacket) (16*GETJSAMPLE(*p++));
+              index=(IndexPacket) (GETJSAMPLE(*p++));
               VerifyColormapIndex(image,index);
               indexes[x]=index;
               *q++=image->colormap[index];
@@ -1116,7 +1116,7 @@ ModuleExport void RegisterJPEGImage(void)
     *entry;
 
   entry=SetMagickInfo("JPEG");
-  entry->thread_support=False;
+  entry->thread_support=False; /* libjpeg is not thread safe */
 #if defined(HasJPEG)
   entry->decoder=(DecoderHandler) ReadJPEGImage;
   entry->encoder=(EncoderHandler) WriteJPEGImage;
@@ -1138,7 +1138,7 @@ ModuleExport void RegisterJPEGImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("JPG");
-  entry->thread_support=False;
+  entry->thread_support=False; /* libjpeg is not thread safe */
 #if defined(HasJPEG)
   entry->decoder=(DecoderHandler) ReadJPEGImage;
   entry->encoder=(EncoderHandler) WriteJPEGImage;
@@ -1427,6 +1427,8 @@ static unsigned int WriteJPEGImage(const ImageInfo *image_info,Image *image)
   /*
     Initialize JPEG parameters.
   */
+  (void) memset(&jpeg_info,0,sizeof(jpeg_info));
+  (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.client_data=(void *) image;
   jpeg_info.err=jpeg_std_error(&jpeg_error);
   jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
